Package: mvnfast
Type: Package
Title: Fast Multivariate Normal and Student's t Methods
Version: 0.2.8
Date: 2023-02-20
Authors@R: c(person("Matteo", "Fasiolo", email = "matteo.fasiolo@gmail.com", role = c("aut", "cre")),
             person("Thijs", " van den Berg", role = c("ctb")))
Maintainer: Matteo Fasiolo <matteo.fasiolo@gmail.com>
Description: Provides computationally efficient tools related to the
    multivariate normal and Student's t distributions. The main functionalities
    are: simulating multivariate random vectors, evaluating multivariate normal or
    Student's t densities and Mahalanobis distances. These tools are very efficient
    thanks to the use of C++ code and of the OpenMP API.
License: GPL (>= 2.0)
URL: https://github.com/mfasiolo/mvnfast/
Imports: Rcpp
Suggests: knitr, rmarkdown, testthat, mvtnorm, microbenchmark, MASS,
        plyr, RhpcBLASctl
LinkingTo: Rcpp, RcppArmadillo, BH
VignetteBuilder: knitr
RoxygenNote: 7.2.2
NeedsCompilation: yes
Packaged: 2023-02-23 11:38:10 UTC; mf15002
Author: Matteo Fasiolo [aut, cre],
  Thijs van den Berg [ctb]
Repository: CRAN
Date/Publication: 2023-02-23 12:40:02 UTC
Built: R 4.4.0; aarch64-apple-darwin21.6.0; 2024-04-29 09:43:41 UTC; unix
