Package: mvnfast
Type: Package
Title: Fast Multivariate Normal and Student's t Methods
Version: 0.2.5
Date: 2018-01-31
Authors@R: c(person("Matteo", "Fasiolo", email = "matteo.fasiolo@gmail.com", role = c("aut", "cre")),
             person("Thijs", " van den Berg", role = c("ctb")))
Maintainer: Matteo Fasiolo <matteo.fasiolo@gmail.com>
Description: Provides computationally efficient tools related to
    the multivariate normal and Student's t distributions. The main functionalities are:
    simulating multivariate random vectors, evaluating multivariate
    normal or Student's t densities and Mahalanobis distances. These tools are very efficient
    thanks to the use of C++ code and of the OpenMP API.
License: GPL (>= 2.0)
URL: https://github.com/mfasiolo/mvnfast, www.sitmo.com
Imports: Rcpp (>= 0.12.0)
Suggests: knitr, rmarkdown, testthat, mvtnorm, microbenchmark, MASS,
        plyr, RhpcBLASctl
LinkingTo: Rcpp, RcppArmadillo, BH
VignetteBuilder: knitr
RoxygenNote: 6.0.1
NeedsCompilation: yes
Packaged: 2018-01-31 13:22:24 UTC; teo
Author: Matteo Fasiolo [aut, cre],
  Thijs van den Berg [ctb]
Repository: CRAN
Date/Publication: 2018-01-31 18:05:28 UTC
Built: R 3.6.3; x86_64-w64-mingw32; 2020-05-04 09:18:10 UTC; windows
Archs: x64
